// calculate tiles

const SIN_PERIOD = 100 / Math.PI;

const RADIUS_OPTS = [ 
	"const",
	"right",
	"left",
	"up",
	"down",
	"right-up",
	"right-down",
	"left-up",
	"left-down",
	"circle-in",
	"circle-out",
	"hyper-in",
	"hyper-out",
	"noise",
	"diamonds",
	"horizontal-bands",
	"vertical-bands",
];

const RADIUS_DESC = {
	"const": "of constant size",
	"right": "getting larger towards the right",
	"left": "getting larger towards the left",
	"up": "getting larger towards the top",
	"down": "getting larger towards the bottom",
	"right-up": "getting larger towards the upper right",
	"right-down": "getting larger towards the lower right",
	"left-up": "getting larger towards the upper left",
	"left-down": "getting larger towards the lower left",
	"circle-in": "forming a circle at the centre",
	"circle-out": "leaving a circular gap at the centre",
	"hyper-in": "forming a starlike pattern at the centre",
	"hyper-out": "leaving a starlike pattern gap in the centre",
	"noise": "of random sizes",
	"diamonds": "in a grid tilted at 45 degrees",
	"horizontal-bands": "in horizontal bands",
	"vertical-bands": "in a vertical bands",
}

function distance(dx, dy) {
	return Math.sqrt(dx ** 2 + dy ** 2);
}


function int_range(v1, v2) {
	const vs = [v1, v2];
	vs.sort((a, b) => a - b);
	const low = Math.floor(vs[0] - 1);
	const high = Math.ceil(vs[1] + 1);
	return [...Array(high - low + 1).keys()].map((i) => i + low);
}

class DotMaker {
	constructor(width, height) {
		this.width = width;
		this.height = height;
		this.wh = 0.5 * (width + height);
		this.cx = 0.5 * width;
		this.cy = 0.5 * height;
	}

	dots(m, n, clip) {
		if( m - n === 0 ) {
			return [];
		}
		const ps = [];
		const is = int_range(-this.width, this.height / m)
		is.map((i) => {
			const js = int_range(m * i + (m - n) * this.width, m * i)
			js.map((j) => {
				const x = (j - m * i) / (m - n);
				const y = m * (x + i);
				if( !clip || (x > 0 && y > 0 && x < this.width && y < this.height) ) {
					ps.push({i:i, j:j, x:x, y:y});
				}
			});
		});
		return ps;
	}

	radius(func) {
	  	switch (func) {
	    	case "const":
	      		return (d, r) => r;
	    	case "right":
	      		return (d, r) => r * d.x / this.width;
	    	case "left":
	      		return (d, r) => r * (this.width - d.x) / this.width;
	    	case "down":
	      		return (d, r) => r * d.y / this.height;
	    	case "up":
	      		return (d, r) => r * (this.height - d.y) / this.height;
	      	case "right-up":
	      		return (d, r) => 0.5 * r * (d.x + this.height - d.y) / this.wh;
	      	case "left-up":
	      		return (d, r) => 0.5 * r * (this.width - d.x + this.height - d.y) / this.wh;
	      	case "right-down":
	      		return (d, r) => 0.5 * r * (d.x + d.y) / this.wh;
	      	case "left-down":
	      		return (d, r) => 0.5 * r * (this.width - d.x + d.y) / this.wh;
	      	case "circle-out":
	      		return (d, r) => 2 * r * distance((d.x - this.cx), (d.y - this.cy)) / this.wh;
	      	case "circle-in":
	      		return (d, r) => 2 * r * (0.5 * this.wh - distance((d.x - this.cx), (d.y - this.cy))) / this.wh;
	      	case "hyper-in":
	      		return (d, r) => r * Math.abs((d.x - this.cx) * (d.y - this.cy)) / this.wh;
	      	case "hyper-out":
	      		return (d, r) => r * (1 - Math.abs((d.x - this.cx) * (d.y - this.cy)) / this.wh);
	      	case "noise":
	      		return (d, r) => r * Math.random();
	      	case "horizontal-bands":
	      		return (d, r) => r * (1 + Math.cos(SIN_PERIOD *(d.y - this.cy)/ this.wh));
	      	case "vertical-bands":
	      		return (d, r) => r * (1 + Math.cos(SIN_PERIOD *(d.x - this.cx)/ this.wh));
	      	case "grid":
	      		return (d, r) => r * (0.5 + 0.5 * (Math.cos(SIN_PERIOD *(d.x - this.cx)/ this.wh) + Math.cos(SIN_PERIOD * (d.y - this.cy)/ this.wh)));
	    	default:
	      		return (d, r) => r;
	  }
	}
}

export { RADIUS_OPTS, RADIUS_DESC, DotMaker };


